"""Common pipetting command base models."""
from pydantic import BaseModel, Field


class BasePipettingRequest(BaseModel):
    """Base class for pipetting requests that interact with wells."""

    pipetteId: str = Field(
        ...,
        description="Identifier of pipette to use for liquid handling.",
    )
    labwareId: str = Field(
        ...,
        description="Identifier of labware to use.",
    )
    wellName: str = Field(
        ...,
        description="Identifier of well to use.",
    )


class BaseLiquidHandlingRequest(BasePipettingRequest):
    """Base class for liquid handling requests."""

    volume: float = Field(
        ...,
        description="Amount of liquid in uL. Must be greater than 0 and less "
                    "than a pipette-specific maximum volume.",
        gt=0,
    )
    flowRate: float = Field(
        ...,
        description="The absolute flow rate in uL/second. Must be greater "
                    "than 0 and less than a pipette-specific maximum flow "
                    "rate.",
        gt=0
    )
